﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Routing;

namespace SimpleRoutingSample
{
    public partial class Form1 : Form
    {
        private static RoutingEngine routingEngine;
        private static RoutingSource RoutingSourceForShortest;
        private static RoutingSource RoutingSourceForFastest;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ShapeFileFeatureSource featureSource = new ShapeFileFeatureSource(@"..\..\SampleData\Austinstreets.shp");

            RoutingSourceForShortest = new RtgRoutingSource(@"..\..\SampleData\Austinstreets.rtg");
            RoutingSourceForFastest = new RtgRoutingSource(@"..\..\SampleData\AustinStreetsForFastest.rtg");
            routingEngine = new RoutingEngine(RoutingSourceForShortest, featureSource);

            RenderMap();

            FindPath();
        }

        private void RadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (rbtShortest.Checked)
            {
                routingEngine.RoutingSource = RoutingSourceForShortest;
            }
            else
            {
                routingEngine.RoutingSource = RoutingSourceForFastest;
            }

            FindPath();
        }

        private void FindPath()
        {
            RoutingLayer routingLayer = (RoutingLayer)((LayerOverlay)winformsMap1.Overlays["RoutingOverlay"]).Layers["RoutingLayer"];

            routingLayer.Routes.Clear();
            LineShape routeLine = new LineShape();
            MultilineShape route = routingEngine.GetRoute(txtStartId.Text, txtEndId.Text).Route;
            MultipointShape multipointShape = route.Lines[0].GetCrossing(route.Lines[1]);
            if (route.Lines[0].Vertices[0] == new Vertex(multipointShape.Points[0]))
            {
                routeLine.Vertices.Add(route.Lines[0].Vertices[route.Lines[0].Vertices.Count - 1]);
            }
            else
            {
                routeLine.Vertices.Add(route.Lines[0].Vertices[0]);
            }

            foreach (LineShape lineShape in route.Lines)
            {
                MergeSegments(routeLine, lineShape);
            }
            MultilineShape newRoute = new MultilineShape();
            newRoute.Lines.Add(routeLine);
            routingLayer.Routes.Add(newRoute);

            winformsMap1.Refresh(winformsMap1.Overlays["RoutingOverlay"]);
        }

        private void RenderMap()
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#e6e5d1"));
            winformsMap1.CurrentExtent = new RectangleShape(-97.763384, 30.299707, -97.712382, 30.259309);

            WorldMapKitWmsDesktopOverlay worldMapKitsOverlay = new WorldMapKitWmsDesktopOverlay();
            winformsMap1.Overlays.Add(worldMapKitsOverlay);

            ShapeFileFeatureSource featureSource = new ShapeFileFeatureSource(@"..\..\SampleData\Austinstreets.shp");
            featureSource.Open();
            RoutingLayer routingLayer = new RoutingLayer();
            routingLayer.RouteStyle.OuterPen.Color = new GeoColor(120, GeoColor.SimpleColors.Blue); 
            routingLayer.StartPoint = featureSource.GetFeatureById(txtStartId.Text, ReturningColumnsType.NoColumns).GetShape().GetCenterPoint();
            routingLayer.EndPoint = featureSource.GetFeatureById(txtEndId.Text, ReturningColumnsType.NoColumns).GetShape().GetCenterPoint();
            LayerOverlay routingOverlay = new LayerOverlay();
            routingOverlay.Layers.Add("RoutingLayer", routingLayer);
            winformsMap1.Overlays.Add("RoutingOverlay", routingOverlay);

            InMemoryFeatureLayer routingExtentLayer = new InMemoryFeatureLayer();
            routingExtentLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColor.SimpleColors.Green));
            routingExtentLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            routingExtentLayer.InternalFeatures.Add(new Feature(new RectangleShape(-97.815409, 30.369949, -97.657999, 30.217922)));
            routingOverlay.Layers.Add("RoutingExtentLayer", routingExtentLayer);

            winformsMap1.Refresh();
        }

        private void MergeSegments(LineShape destinationLine, LineShape sourceLine)
        {
            if (destinationLine.Vertices[destinationLine.Vertices.Count - 1] == sourceLine.Vertices[0])
            {
                for (int i = 1; i < sourceLine.Vertices.Count; i++)
                {
                    destinationLine.Vertices.Add(sourceLine.Vertices[i]);
                }
            }
            else
            {
                for (int i = sourceLine.Vertices.Count - 2; i >= 0; i--)
                {
                    destinationLine.Vertices.Add(sourceLine.Vertices[i]);
                }
            }
        }
    }
}
